#include "brain.h"
// #include "3dviewer.h"

#include <assert.h>
#include <math.h>

// ********************************************************************
// ************************** CBrain **********************************
// ********************************************************************
CBrain::CBrain(
	       MyBitString igenotype
	       )
{
    assert(igenotype.length() == GENOTYPE_LENGTH);
    assert(NR_OF_CONNECTABLE_CELLS == NR_OF_SENSORS + NR_OF_WHEELS + NR_OF_NEURONS);

    
    genotype = igenotype;

    int i=0;
    for (int j=0; j<NR_OF_SENSORS; j++) cells[i++] = &sensors[j];
    for (int j=0; j<NR_OF_WHEELS; j++)  cells[i++] = &wheels[j];
    for (int j=0; j<NR_OF_NEURONS; j++) cells[i++] = &neurons[j];
    assert(i == NR_OF_CONNECTABLE_CELLS);

    for (int i=0; i<NR_OF_CONNECTABLE_CELLS; i++)
	for (int j=0; j<NR_OF_CONNECTABLE_CELLS; j++)
	    ANN[i][j] = 0.0;









    //printf("building ANN\n");
    // ********************************************************************
    // build up the ANN!!!
    // ********************************************************************


    CGrowing_Grid *growing_grid; // we have to make it dynamic. otherwhise -> segfault :(
    growing_grid = new CGrowing_Grid;


    CGrowing_Axon_List growing_axonlist;
    
    CGrowing_Axon_Cell*     growing_sensors[NR_OF_SENSORS];
    CGrowing_Both_Cell*     growing_neurons[NR_OF_NEURONS];
    CGrowing_Dentrite_Cell* growing_wheels[NR_OF_WHEELS];

    int current_index = 0; // so the ANN can be set correctly later;
    int geno_type_pos = 0; // where we are in the genotype at the moment;

    
    assert(GENOTYPE_LENGTH == GENOTYPE_LENGTH_SENSORS * NR_OF_SENSORS + 
	                      GENOTYPE_LENGTH_WHEELS * NR_OF_WHEELS + 
                              GENOTYPE_LENGTH_NEURONS * NR_OF_NEURONS + 
                              GENOTYPE_LENGTH_GUIDANCECELLS * NR_OF_GUIDANCECELLS);

    // initialize sensors
    for (int i=0; i<NR_OF_SENSORS; i++)
	{
	    assert(GENOTYPE_LENGTH_AXONGROWING == GENOTYPE_LENGTH_SENSORS);

	    growing_sensors[i] = new CGrowing_Axon_Cell(
							MyBitString(
								    genotype.at( //AxonGrowing
										geno_type_pos, 
										GENOTYPE_LENGTH_AXONGROWING
										)
								    ),
							get_sensor_pos(i),
							current_index++,
							growing_axonlist
							);
	    geno_type_pos += GENOTYPE_LENGTH_SENSORS;
	};
    // initialize wheels
    for (int i=0; i<NR_OF_WHEELS; i++)
	{
	    assert(GENOTYPE_LENGTH_WHEELS == GENOTYPE_LENGTH_AXONATTRACTING);

	    growing_wheels[i] = new CGrowing_Dentrite_Cell(
							   get_wheel_pos(i),
							   current_index++
							   );
	    growing_grid->add_concentration(
					    MyBitString(
							genotype.at( //AxonAttracting
								    geno_type_pos, 
								    GENOTYPE_LENGTH_AXONATTRACTING
								    )
							),
					    get_wheel_pos(i)
					    );
	    geno_type_pos += GENOTYPE_LENGTH_WHEELS;
	};
    // initialize neurons
    for (int i=0; i<NR_OF_NEURONS; i++)
	{
	    // in the genotype:
	    // AxonGrowing, AxonAcctracting, Position, Threshold

	    assert(GENOTYPE_LENGTH_NEURONS == GENOTYPE_LENGTH_AXONGROWING + GENOTYPE_LENGTH_AXONATTRACTING + GENOTYPE_LENGTH_POSITION + GENOTYPE_LENGTH_THRESHOLD);

	    growing_neurons[i] = new CGrowing_Both_Cell(
							MyBitString(
								    genotype.at( // AxonGrowing
										geno_type_pos, 
										GENOTYPE_LENGTH_AXONGROWING
										)
								    ),
							C3dpoint(MyBitString(
									     genotype.at( // Position
											 geno_type_pos + 
											 GENOTYPE_LENGTH_AXONGROWING + 
											 GENOTYPE_LENGTH_AXONATTRACTING, 
											 GENOTYPE_LENGTH_POSITION
											 )
									     )
								 ),
							current_index++,
							growing_axonlist
							);
	    
	    growing_grid->add_concentration(
					    MyBitString(
							genotype.at( // AxonAttracting
								    geno_type_pos +
								    GENOTYPE_LENGTH_AXONGROWING,
								    GENOTYPE_LENGTH_AXONATTRACTING)
							),
					    C3dpoint(MyBitString(
								 genotype.at( // Position
									     geno_type_pos + 
									     GENOTYPE_LENGTH_AXONGROWING + 
									     GENOTYPE_LENGTH_AXONATTRACTING, 
									     GENOTYPE_LENGTH_POSITION
									     )
								 )
						     )
					    );
	    
	    neurons[i].set_threshold(
				     (float)(genotype.value_at( // threshold
							       geno_type_pos + 
							       GENOTYPE_LENGTH_AXONGROWING + 
							       GENOTYPE_LENGTH_AXONATTRACTING +
							       GENOTYPE_LENGTH_POSITION,
							       GENOTYPE_LENGTH_THRESHOLD
							       )
					     )
				     / DIVIDE_BITSTRING_THRESHOLD
				     );
	    
	    geno_type_pos += GENOTYPE_LENGTH_NEURONS;
	};

    // initialize guidance cells
    for (int i=0; i<NR_OF_GUIDANCECELLS; i++)
	{
	    assert(GENOTYPE_LENGTH_GUIDANCECELLS == GENOTYPE_LENGTH_AXONATTRACTING + GENOTYPE_LENGTH_POSITION);
	    
	    growing_grid->add_concentration(
					    MyBitString(
							genotype.at( // AxonAttracting
								    geno_type_pos,
								    GENOTYPE_LENGTH_AXONATTRACTING)
							),
					    C3dpoint(MyBitString(
								 genotype.at( // Position
									     geno_type_pos + 
									     GENOTYPE_LENGTH_AXONATTRACTING, 
									     GENOTYPE_LENGTH_POSITION
									     )
								 )
						     )
					    );
	    
	    geno_type_pos += GENOTYPE_LENGTH_GUIDANCECELLS;
	};

    assert(GENOTYPE_LENGTH == geno_type_pos);

    

//    C3D_Viewer_Concentration cviewer;
//     cviewer.show(*growing_grid);


    //printf("-------------------growing-------------------\n");

    // grow the net

//C3D_Viewer_Occupied oviewer;
//oviewer.show(*growing_grid);
    while ( growing_axonlist.grow(*growing_grid) ) 
;//	oviewer.show(*growing_grid);


//     int frei=0;
//     int voll=0;
//     for (int x=0; x<CUBE_SIZE; x++)
// 	for (int y=0; y<CUBE_SIZE; y++)
// 	    for (int z=0; z<CUBE_SIZE; z++)
// 		if (growing_grid->get_occupied(x,y,z))
// 		    voll++;
// 		else
// 		    frei++;
    





//printf("voll %d; frei %d\n", voll, frei);

    //printf("get weigths\n");
    // for all dendrites (wheels, neurons) get_weights
    for (int i=0; i<NR_OF_WHEELS; i++)
	{
	    growing_wheels[i]->get_weight_vector(
						 ANN[ growing_wheels[i]->get_number() ],
						 growing_axonlist
						 );
	};
    for (int i=0; i<NR_OF_NEURONS; i++)
	{
	    growing_neurons[i]->get_weight_vector(
						  ANN[ growing_neurons[i]->get_number() ],
						  growing_axonlist
						  );
	};



/*    for (int x=0; x<CUBE_SIZE; x++)
      {
	for (int y=0; y<CUBE_SIZE; y++)
	  printf("%c", growing_grid->get_occupied(x,y,40) ? '1': ' ');
	printf("\n");
      };

*/

/*
    for (int x=0; x<CUBE_SIZE; x++)
      {
	for (int y=0; y<CUBE_SIZE; y++)
	  printf("%f\t", growing_grid->get_concentration(x,y,41,7));
	printf("\n");
      };

*/

//     for (int dest=0; dest<NR_OF_CONNECTABLE_CELLS; dest++)
//       {
// 	float sum_of_stimuli = 0.0;
	
// 	for (int source=0; source<NR_OF_CONNECTABLE_CELLS; source++)
// 	  {
// 	    printf("%f  ",ANN[dest][source]);
// 	  };
	
// 	printf("\n");
//       };


    


    // free the growing_cells
    for (int i=0; i<NR_OF_SENSORS; i++) { delete growing_sensors[i]; };
    for (int i=0; i<NR_OF_WHEELS; i++)  { delete growing_wheels[i];  };
    for (int i=0; i<NR_OF_NEURONS; i++) { delete growing_neurons[i]; };

    delete growing_grid ;
};

CBrain::~CBrain()
{
    // nothing to be done
};

void CBrain::load_sensors(
			  float input[NR_OF_SENSORS]
			  )
{
    for (int i=0; i<NR_OF_SENSORS; i++)
	{
//printf("%i  %f\n", i, input[i]);
	    sensors[i].incoming_stimulus(input[i]);
	    sensors[i].process();
	};
};

void CBrain::compute()
{
    for (int dest=0; dest<NR_OF_CONNECTABLE_CELLS; dest++)
	{
	    float sum_of_stimuli = 0.0;
	    
	    for (int source=0; source<NR_OF_CONNECTABLE_CELLS; source++)
		{
		    sum_of_stimuli += cells[source]->get_out_value() * ANN[dest][source];
		};
	    cells[dest]->incoming_stimulus(sum_of_stimuli);
	};

    for (int i=0; i<NR_OF_CONNECTABLE_CELLS; i++)
	cells[i]->process();
};

void CBrain::get_wheels(
			float output[NR_OF_WHEELS]
			)
{
    for (int i=0; i<NR_OF_WHEELS; i++)
	output[i]=wheels[i].get_out_value();
};

void CBrain::get_genotype(
			  MyBitString &igenotype
			  )
{
    igenotype = genotype;
};
