#ifndef __CONSTANTS_H
#define __CONSTANTS_H

#define PI                                                 3.1415926535



/*
           T H E     G E N O T Y P E

SENSORS: 4 *                            NR_OF_SENSORS
       Axongrowingpart:
               4 * (3b + 8b)            NR_OF_SUBSTANCES_TO_FOLLOW * GENOTYPE_LENGTH_SUBSTANCE_TO_FOLLOW
                                                                       (=GENOTYPE_LENGTH_ONE_SUBSTANCE + GENOTYPE_LENGTH_SUBSTANCE_THRESHOLD)
               8b                       GENOTYPE_LENGTH_INHIBITORY_PROPABILITY
	       7b                       GENOTYPE_LENGTH_CONNECTIVITY
               2b                       GENOTYPE_LENGTH_TERMINATE_GROWTH

WHEELS: 2 *                             NR_OF_WHEELS
       Axonattractingpart:
               3b                       GENOTYPE_LENGTH_ONE_SUBSTANCE
	       7b                       GENOTYPE_LENGTH_SUBSTANCE_INITIAL_CONCENTRATION

NEURONS: 10 *                           NR_OF_NEURONS
       Axongrowingpart:
               4 * (3b + 9b)            NR_OF_SUBSTANCES_TO_FOLLOW * GENOTYPE_LENGTH_SUBSTANCE_TO_FOLLOW
                                                                       (=GENOTYPE_LENGTH_ONE_SUBSTANCE + GENOTYPE_LENGTH_SUBSTANCE_THRESHOLD)
               8b                       GENOTYPE_LENGTH_INHIBITORY_PROPABILITY
	       7b                       GENOTYPE_LENGTH_CONNECTIVITY
               2b                       GENOTYPE_LENGTH_TERMINATE_GROWTH

       Axonattractingpart:
               3b                       GENOTYPE_LENGTH_ONE_SUBSTANCE
	       7b                       GENOTYPE_LENGTH_SUBSTANCE_INITIAL_CONCENTRATION

       Position
               3 * 6b                   (3 * GENOTYPE_LENGTH_POSITION_ONE_COORDINATE)
	     
       Threshold
               8b                       GENOTYPE_LENGTH_THRESHOLD

GUIDECELLS: 10 *                        NR_OF_GUIDANCECELLS
       Axonattractingpart:
               3b                       GENOTYPE_LENGTH_ONE_SUBSTANCE
	       7b                       GENOTYPE_LENGTH_SUBSTANCE_INITIAL_CONCENTRATION

       Position
               3 * 6b                   (3 * GENOTYPE_LENGTH_POSITION_ONE_COORDINATE)

*/    




// this allows 16 substances
#define GENOTYPE_LENGTH_ONE_SUBSTANCE                      3
#define NR_OF_SUBSTANCES                                   8 // (2^GENOTYPE_LENGTH_ONE_SUBSTANCE)


// a concentration threshold may be between 0.0 and 2.0
#define GENOTYPE_LENGTH_SUBSTANCE_THRESHOLD                8
#define DIVIDE_GENOTYPE_SUBSTANCE_THRESHOLD                128.0
#define NR_OF_SUBSTANCES_TO_FOLLOW                         4
// between 0 and 1.0 
#define GENOTYPE_LENGTH_INHIBITORY_PROPABILITY             8
#define DIVIDE_BITSTRING_INHIBITORY_PROPABILITY            1.0

// between 0.0 and 12.7
#define GENOTYPE_LENGTH_CONNECTIVITY                       7
#define DIVIDE_BITSTRING_CONNECTIVITY                      10.0

#define GENOTYPE_LENGTH_TERMINATE_GROWTH                   2
#define GENOTYPE_LENGTH_SUBSTANCE_TO_FOLLOW                (GENOTYPE_LENGTH_ONE_SUBSTANCE + GENOTYPE_LENGTH_SUBSTANCE_THRESHOLD)
#define GENOTYPE_LENGTH_AXONGROWING                        (NR_OF_SUBSTANCES_TO_FOLLOW * GENOTYPE_LENGTH_SUBSTANCE_TO_FOLLOW + GENOTYPE_LENGTH_INHIBITORY_PROPABILITY + GENOTYPE_LENGTH_CONNECTIVITY + GENOTYPE_LENGTH_TERMINATE_GROWTH )


// an initial concentration may be between 0 and 1.0
#define GENOTYPE_LENGTH_SUBSTANCE_INITIAL_CONCENTRATION    7
#define DIVIDE_BITSTRING_INITIAL_CONCENTRATION             127.0 // ((float)(2^GENOTYPE_LENGTH_SUBSTANCE_INITIAL_CONCENTRATION)-1)
#define GENOTYPE_LENGTH_AXONATTRACTING                     (GENOTYPE_LENGTH_ONE_SUBSTANCE + GENOTYPE_LENGTH_SUBSTANCE_INITIAL_CONCENTRATION)

// the inner world is a cube wich is 256*256*256 in size
#define GENOTYPE_LENGTH_POSITION_ONE_COORDINATE            6
#define CUBE_SIZE                                          64 // (2^GENOTYPE_LENGTH_POSITION_ONE_COORDINATE)
#define GENOTYPE_LENGTH_POSITION                           (3 * GENOTYPE_LENGTH_POSITION_ONE_COORDINATE)

// a threshold may be between 0 and 255
#define GENOTYPE_LENGTH_THRESHOLD                          8
#define DIVIDE_BITSTRING_THRESHOLD                         127.0 // ((float)(2^(GENOTYPE_LENGTH_THRESHOLD-1)))

#define GENOTYPE_LENGTH_SENSORS                            GENOTYPE_LENGTH_AXONGROWING
#define GENOTYPE_LENGTH_GUIDANCECELLS                      (                              GENOTYPE_LENGTH_AXONATTRACTING + GENOTYPE_LENGTH_POSITION)
#define GENOTYPE_LENGTH_NEURONS                            (GENOTYPE_LENGTH_AXONGROWING + GENOTYPE_LENGTH_AXONATTRACTING + GENOTYPE_LENGTH_POSITION + GENOTYPE_LENGTH_THRESHOLD)
#define GENOTYPE_LENGTH_WHEELS                                                            GENOTYPE_LENGTH_AXONATTRACTING


#define NR_OF_EYES                                         2
#define NR_OF_WHISKERS                                     2
#define NR_OF_SENSORS                                      (NR_OF_EYES+NR_OF_WHISKERS)
#define NR_OF_WHEELS                                       2
#define NR_OF_NEURONS                                      10
#define NR_OF_GUIDANCECELLS                                10
#define NR_OF_AXON_ATTRACTING                              (NR_OF_WHEELS + NR_OF_GUIDANCECELLS + NR_OF_NEURONS)

#define NR_OF_CONNECTABLE_CELLS                            (NR_OF_SENSORS + NR_OF_WHEELS + NR_OF_NEURONS)

#define GENOTYPE_LENGTH                                    (NR_OF_SENSORS*GENOTYPE_LENGTH_SENSORS + NR_OF_WHEELS*GENOTYPE_LENGTH_WHEELS + NR_OF_NEURONS*GENOTYPE_LENGTH_NEURONS + NR_OF_GUIDANCECELLS*GENOTYPE_LENGTH_GUIDANCECELLS)
#define GENOTYPE_LENGTH_WITH_TERM                          (GENOTYPE_LENGTH + 1)


// ANN proccessing constants
#define OUT_IF_THRESHOLD_EXCEEDED                          1.0
#define IN_IF_THRESHOLD_EXCEEDED                           0.0

#define OUT_IF_THRESHOLD_NOT_EXCEEDED                      0.0
#define IN_IF_THRESHOLD_NOT_EXCEEDED(old_in)               (old_in*0.95)

// energy household constants
#define ENERGY_INITIAL                                     200
#define ENERGY_EAT                                         20
#define ENERGY_CRASH                                       -10
#define ENERGY_LIVECYCLE                                   -1

// moving constants
#define WHEEL_CIRCUMFERENCE                                2.0
// so that if one wheel does not move at all and one wheel moves at full power the creature turns 90 degrees or pi/2 rad.
#define WHEELS_DISTANCE                                    (10.0*2.0/PI)

// axon growth constants
#define AXON_LENGTH_PENALTY                                0.01
#define AXON_DISTANCE_PENALTY                              1.0




//the function to determine the concentrations can be found in CGrowing_Grid::add_concentration
//the positions of sensors and wheels in the inner world can be set in 3dpoint.cc





#define POPULATION_SIZE                                     100


#define REPRODUCE_GROUND_PROBABILITY                        0.0
#define REPRODUCE_PROBABILITY_EXPONENT                      0.333
#define REPRODUCE_PROBABILITY_ADD                           10
#define REPRODUCE_PROBABILITY_DIVIDE                        0.1

#define REPRODUCE_PROBABILITY_BITFLIP                       (1/GENOTYPE_LENGTH)
#define REPRODUCE_PROBABILITY_CROSSOVER                     0.2


//external world constants
#define FOOD_PROBABILITY                                    0.1

#define OBSTACLE                                            -1
#define EMPTY                                               0
#define FOOD                                                1

#define EYE_STRENGTH                                        8
#define EYE_WIDTH                                           (30 *2*PI/360) // 30 deg left + right =60 deg
#define EYE_START                                           (-15 *2*PI/360)
#define EYE_STEP                                            (30 *2*PI/360)
#define WHISKER_LENGTH                                      4
#define WHISKER_WIDTH                                       (45 *2*PI/360) // 90 deg
#define WHISKER_START                                       (-45 *2*PI/360)
#define WHISKER_STEP                                        (90 *2*PI/360)





#define MAP_FILE                                            "field.map"
#endif //__CONSTANTS_H
