#include "creature.h"

#include <assert.h>
#include <math.h>

// ********************************************************************
// ************************** CCreature *******************************
// ********************************************************************
CCreature::CCreature(
		     MyBitString igenotype,
		     float ix,
		     float iy,
		     float idir
		     ) : CBrain(igenotype)
{
    // brain building is done by parent
    energy = ENERGY_INITIAL;
    xpos = ix;
    ypos = iy;
    direction = idir;
};

CCreature::~CCreature()
{
    // nothing to be done
};

void CCreature::move(
		     float input[NR_OF_SENSORS]
		     )
{
    float wheels[NR_OF_WHEELS];
    float moved_distance[NR_OF_WHEELS];

    load_sensors(input);
    compute();
    get_wheels(wheels);
    

    assert(WHEEL_CIRCUMFERENCE != 0);
    for (int i=0; i<NR_OF_WHEELS; i++)
	moved_distance[i] = wheels[i] * WHEEL_CIRCUMFERENCE;


    //compute movement
    assert(NR_OF_WHEELS == 2); 
    assert(WHEELS_DISTANCE != 0);

    float delta_angle = ((moved_distance[1] - moved_distance[0]) / WHEELS_DISTANCE);
    float forward;
    float right;

    if (delta_angle==0.0)
	{
	    forward = moved_distance[0];
	    right = 0;
	}
    else
	{
	    float radius = moved_distance[0] / delta_angle  +  WHEELS_DISTANCE / 2.0;
	    forward = sin(delta_angle) * radius;
	    right   = cos(delta_angle) * radius - radius;
	};

    float delta_y = cos(direction) * forward  +  sin(direction) * right;
    float delta_x = cos(direction) * right    -  sin(direction) * forward;


    ypos += delta_y;
    xpos += delta_x;
    direction += delta_angle;

    while (direction >= 2*PI)
	direction -= 2*PI;
    
    while (direction < 0)
	direction += 2*PI;
};



float CCreature::get_x()
{
    return xpos;
};

float CCreature::get_y()
{
    return ypos;
};

float CCreature::get_direction()
{
    return direction;
};

void CCreature::set_x(float in)
{
    xpos = in;
};

void CCreature::set_y(float in)
{
    ypos = in;
};

void CCreature::set_direction(float in)
{
    direction = in;

    while (direction >= 2*PI)
	direction -= 2*PI;
    
    while (direction < 0)
	direction += 2*PI;
};

void CCreature::live_cycle()
{
    energy += ENERGY_LIVECYCLE;
};

void CCreature::eat()
{
    energy += ENERGY_EAT;
};

void CCreature::crash()
{
    energy += ENERGY_CRASH;
};

bool CCreature::alive()
{
    return energy>0;
}
