#ifndef __GROWING_CELL_H
#define __GROWING_CELL_H


#include "MyBitString.h"
#include <stdlib.h>

#include "constants.h"
#include "growing.h"
#include "3dpoint.h"
 
class CGrowing_Cell {
protected:
    C3dpoint pos;
    int number;
    
public:
    CGrowing_Cell(
		  C3dpoint ipos,
		  int inumber
		  );
    ~CGrowing_Cell() {};
    
    int get_number();
};

class CGrowing_Axon_Cell: public CGrowing_Cell {
private:
    int substances_to_follow[NR_OF_SUBSTANCES_TO_FOLLOW];
    float thresholds[NR_OF_SUBSTANCES_TO_FOLLOW];
    float hairiness;
    float inhibitory_probability;
    int max_inactive_turns;
    
public:
    CGrowing_Axon_Cell(
		       MyBitString genotype,
		       C3dpoint ipos,
		       int inumber,
		       CGrowing_Axon_List &axonlist
		       );
    ~CGrowing_Axon_Cell() {};
    
    int get_number();

    int get_substance_from_substances_to_follow(
						int substance_number
						);
    float get_threshold_from_substances_to_follow(
						  int substance_number
						  );
    
    float get_hairiness();
    
    bool get_inhibitory();

    int get_max_inactive_turns();
};

class CGrowing_Dentrite_Cell: public CGrowing_Cell {
public:
    CGrowing_Dentrite_Cell(
			   C3dpoint ipos,
			   int inumber
			   ): CGrowing_Cell(ipos, inumber) {};
    ~CGrowing_Dentrite_Cell() {};
    
    int get_number();

    void get_weight_vector(
			   float weights[],
			   CGrowing_Axon_List &axon_list
			   );
};

class CGrowing_Both_Cell: public CGrowing_Axon_Cell, public CGrowing_Dentrite_Cell {
public:
    CGrowing_Both_Cell(
		       MyBitString genotype,
		       C3dpoint ipos,
		       int inumber,
		       CGrowing_Axon_List &axonlist
		       ): CGrowing_Axon_Cell(genotype, ipos, inumber, axonlist), CGrowing_Dentrite_Cell(ipos, inumber) {};
    ~CGrowing_Both_Cell() {};

    int get_number();
    
};




#endif __GROWING_CELL_H
