#include "growinggrid.h"

#include <math.h>
#include <assert.h>
// ********************************************************************
// ************************** CGrowing_Grid ***************************
// ********************************************************************
CGrowing_Grid::CGrowing_Grid()
{
    for (int x=0; x<CUBE_SIZE; x++)
	for (int y=0; y<CUBE_SIZE; y++)
	    for (int z=0; z<CUBE_SIZE; z++)
		{
		    grid[x][y][z].occupied = false;
		    for (int i=0; i<NR_OF_SUBSTANCES; i++)
		      grid[x][y][z].initialized[i] = false;
		};
    akt_axon_attracting = 0;

};

CGrowing_Grid::~CGrowing_Grid()
{
    // nothing to be done
}

void CGrowing_Grid::add_concentration(
				      C3dpoint pos,
				      float c,
				      int substance
				      )
{
    assert(c>=0.0); assert(c <= 1.0);
    assert(substance >= 0); 
    assert(substance < NR_OF_SUBSTANCES);

/*
    for (int x=0; x<CUBE_SIZE; x++)
	for (int y=0; y<CUBE_SIZE; y++)
	    for (int z=0; z<CUBE_SIZE; z++)
		{
		    float dist = sqrt( (float)(pow(x-pos.x(),2) + pow(y-pos.y(),2) + pow(z-pos.z(),2)));
		    float conc = dist == 0 ? c : c*pow(dist, -1);
			
		    grid[x][y][z].concentration[substance] += conc;
		};
*/

    axon_attracting[akt_axon_attracting  ].init_concentration = c;
    axon_attracting[akt_axon_attracting  ].substance = substance;
    axon_attracting[akt_axon_attracting  ].x = pos.x();
    axon_attracting[akt_axon_attracting  ].y = pos.y();
    axon_attracting[akt_axon_attracting++].z = pos.z();
    
    grid[pos.x()][pos.y()][pos.z()].occupied = true;
};

void CGrowing_Grid::add_concentration(
				      MyBitString genotype,
				      C3dpoint pos
				      )
{
    assert(genotype.length() == GENOTYPE_LENGTH_AXONATTRACTING);

    add_concentration(
		      pos,
		      (float)(genotype.value_at(GENOTYPE_LENGTH_ONE_SUBSTANCE, GENOTYPE_LENGTH_SUBSTANCE_INITIAL_CONCENTRATION))
		      / DIVIDE_BITSTRING_INITIAL_CONCENTRATION,
		      genotype.value_at(0, GENOTYPE_LENGTH_ONE_SUBSTANCE)
		      );
};

void CGrowing_Grid::set_occupied(
				 int x,
				 int y,
				 int z)
{
    assert(x >= 0); assert(x < CUBE_SIZE);
    assert(y >= 0); assert(y < CUBE_SIZE);
    assert(z >= 0); assert(z < CUBE_SIZE);
    assert(grid[x][y][z].occupied == false);

    grid[x][y][z].occupied = true;
};

bool CGrowing_Grid::get_occupied(
				 int x,
				 int y,
				 int z)
{
    assert(x >= 0); assert(x < CUBE_SIZE);
    assert(y >= 0); assert(y < CUBE_SIZE);
    assert(z >= 0); assert(z < CUBE_SIZE);

    return (grid[x][y][z].occupied);
};


float CGrowing_Grid::get_concentration(
				       int x,
				       int y,
				       int z,
				       int substance
				       )
{
    assert(x >= 0); assert(x < CUBE_SIZE);
    assert(y >= 0); assert(y < CUBE_SIZE);
    assert(z >= 0); assert(z < CUBE_SIZE);
    assert(substance >= 0); 
    assert(substance < NR_OF_SUBSTANCES);


// all add_concentration must already have taken place!
    assert(akt_axon_attracting == NR_OF_AXON_ATTRACTING);

    if (!grid[x][y][z].initialized[substance])
	{
	  grid[x][y][z].concentration[substance] = 0.0;


	    for (int i=0; i<NR_OF_AXON_ATTRACTING; i++)
		{
		    if (axon_attracting[i].substance==substance)
			{
			    float dist = sqrt( (float)(pow(x-axon_attracting[i].x,2) + 
						       pow(y-axon_attracting[i].y,2) + 
						       pow(z-axon_attracting[i].z,2)));
			    float conc = dist == 0 ? axon_attracting[i].init_concentration : axon_attracting[i].init_concentration*pow(dist, -1);
			    
			    grid[x][y][z].concentration[substance] += conc;
			}
		};

	    grid[x][y][z].initialized[substance] = true;
	};

    return (grid[x][y][z].concentration[substance]);
};
