#include "worldviewer.h"


CWorld_Viewer::CWorld_Viewer()
{
    initialized = false;
};

CWorld_Viewer::~CWorld_Viewer()
{
    if (initialized)
	XDestroyWindow(display, win);
};

void CWorld_Viewer::init_windows(int xs, int ys)
{
    /* Attempt to connect to the server and use the default screen */
    if ((display = XOpenDisplay(NULL)) == NULL) {
	perror("Can't connect to server");
	exit(1);
    }
    screen = DefaultScreen(display);
    
    /* Create a window */
    win = XCreateSimpleWindow(display, RootWindow(display, screen),
			      100, 200, xs*6, ys*6, 1, BlackPixel(display, screen),
			      WhitePixel(display, screen));
    
    initialized= true;
};

void CWorld_Viewer::show(int &xs, int &ys, int **&world, CCreature *creatures[])
{

  //    XKeyEvent *kev;
    
    if (!initialized)
	init_windows(xs,ys);
    
    /* Set up which event types the window will handle */
    XSelectInput(display, win, ExposureMask | ButtonPressMask);
    
    /* Graphics contexts */
    setUpGCs();
    
    /* Map the window in order for it to be displayed */
    XMapWindow(display, win);
    
    
    drawInToWindow(xs, ys, world, creatures);
    
    /* Enter an infinite loop, awaiting and responding to the chosen events */
    while (0) {
	XNextEvent(display, &event);
	switch (event.type) {
	case Expose: drawInToWindow(xs, ys, world, creatures); break; /* Draw/re-draw everything */
	case ButtonPress: return; break; /* Quit on mouse button press */
	default: break;
	};
    };
};

void CWorld_Viewer::setUpGCs()
    /* Set up 2 graphics context resources in the display server */
    /* gc1 for message and rectangle; gc2 for ellipse */
{
    
    gc1 = XCreateGC(display, win, 0, &gs1); /* 0 -> use defaults */
    gc2 = XCreateGC(display, win, 0, &gs2); /* 0 -> use defaults */
    /* Now override some of the defaults */
    XSetForeground(display, gc1, BlackPixel(display, screen));
    XSetBackground(display, gc1, WhitePixel(display, screen));
};

void CWorld_Viewer::rectangle(int x, int y)
{
	    XDrawPoint(display, win, gc1, x*6  , y*6  );
	    XDrawPoint(display, win, gc1, x*6+1, y*6  );
	    XDrawPoint(display, win, gc1, x*6+2, y*6  );
	    XDrawPoint(display, win, gc1, x*6+3, y*6  );
	    XDrawPoint(display, win, gc1, x*6+4, y*6  );
	    XDrawPoint(display, win, gc1, x*6+5, y*6  );

	    XDrawPoint(display, win, gc1, x*6  , y*6+1);
	    XDrawPoint(display, win, gc1, x*6+1, y*6+1);
	    XDrawPoint(display, win, gc1, x*6+2, y*6+1);
	    XDrawPoint(display, win, gc1, x*6+3, y*6+1);
	    XDrawPoint(display, win, gc1, x*6+4, y*6+1);
	    XDrawPoint(display, win, gc1, x*6+5, y*6+1);

	    XDrawPoint(display, win, gc1, x*6  , y*6+2);
	    XDrawPoint(display, win, gc1, x*6+1, y*6+2);
	    XDrawPoint(display, win, gc1, x*6+2, y*6+2);
	    XDrawPoint(display, win, gc1, x*6+3, y*6+2);
	    XDrawPoint(display, win, gc1, x*6+4, y*6+2);
	    XDrawPoint(display, win, gc1, x*6+5, y*6+2);

	    XDrawPoint(display, win, gc1, x*6  , y*6+3);
	    XDrawPoint(display, win, gc1, x*6+1, y*6+3);
	    XDrawPoint(display, win, gc1, x*6+2, y*6+3);
	    XDrawPoint(display, win, gc1, x*6+3, y*6+3);
	    XDrawPoint(display, win, gc1, x*6+4, y*6+3);
	    XDrawPoint(display, win, gc1, x*6+5, y*6+3);

	    XDrawPoint(display, win, gc1, x*6  , y*6+4);
	    XDrawPoint(display, win, gc1, x*6+1, y*6+4);
	    XDrawPoint(display, win, gc1, x*6+2, y*6+4);
	    XDrawPoint(display, win, gc1, x*6+3, y*6+4);
	    XDrawPoint(display, win, gc1, x*6+4, y*6+4);
	    XDrawPoint(display, win, gc1, x*6+5, y*6+4);

	    XDrawPoint(display, win, gc1, x*6  , y*6+5);
	    XDrawPoint(display, win, gc1, x*6+1, y*6+5);
	    XDrawPoint(display, win, gc1, x*6+2, y*6+5);
	    XDrawPoint(display, win, gc1, x*6+3, y*6+5);
	    XDrawPoint(display, win, gc1, x*6+4, y*6+5);
	    XDrawPoint(display, win, gc1, x*6+5, y*6+5);

};

void CWorld_Viewer::drawInToWindow(int &xs, int &ys, int **&world, CCreature *creatures[])
{


    for (int i= 0; i<ys; i++)
	for (int j= 0; j<xs; j++) {
	    switch (world[i][j]) {
	    case OBSTACLE:
		XSetForeground(display, gc1, 1240);
		break;
	    case FOOD:
		XSetForeground(display, gc1, 20);
		break;
	    default: 
		XSetForeground(display, gc1, BlackPixel(display, screen));
		break;
	    };
	    rectangle(j,i);
	};

  
  XSetForeground(display, gc1, WhitePixel(display, screen));
  for (int i= 0; i<POPULATION_SIZE; i++) 
      if (creatures[i]->alive())
	    rectangle((int)creatures[i]->get_x()  , (int)creatures[i]->get_y());
};
